  --*----------------------------------------------------------------*
  --*  screenio.e  screen read/display routines                      *
  --*----------------------------------------------------------------*
without warning
  include get.e
  include graphics.e
  include image.e

  global constant ROW     = 1, COL       = 2, FIELDLEN   = 3,
                  CURRCOL = 4, PROTECTED = 5, FIELDCOLOR = 6, VALUE = 7
  sequence saved_cursor

procedure SetCursor(integer row, integer col)
  integer fore, back
  -- if foreground = background, then text is invisible for password

  position (row, col)
  saved_cursor = save_text_image({row, col}, {row, col})
  fore = remainder(saved_cursor[1][2], 16)      -- foreground color
  back = floor(saved_cursor[1][2] / 16)         -- background color
  text_color(WHITE)  bk_color(BLUE)
  if saved_cursor [1][1] = 32 or saved_cursor [1][1] = 219 or
    fore = back then                        -- foreground = background
    position (row, col)  puts(1, "")
  elsif saved_cursor [1][1] > 32 and saved_cursor [1][1] < 128 then
    text_color(BLUE)  bk_color(WHITE)
    position (row, col)  puts(1, saved_cursor [1][1])
  end if

  text_color(BRIGHT_WHITE)  bk_color(CYAN)
  position (25, 55)  printf(1, " %d %d  ", {row, col})
  text_color(WHITE)   bk_color(BLUE)
  position (row, col)

end procedure

procedure ResetCursor(integer row, integer col)
  text_color(WHITE)  bk_color(BLUE)
  position (row, col)
  display_text_image({row, col}, saved_cursor)

end procedure

global procedure RefreshScreen(sequence fields)
  for i = 1 to length(fields) do
    text_color(fields[i][FIELDCOLOR])
    position(fields[i][ROW], fields[i][COL])
    puts(1, repeat(' ', fields[i][FIELDLEN]))  -- fill with spaces
    position(fields[i][ROW], fields[i][COL])   -- reposition to start
    printf(1, "%s", {fields[i][VALUE]})
  end for
end procedure

procedure ValidateTag(object temp)
  if not sequence(temp) then
    puts(1, "\n  include screenio.e function SetupFields - error: \n")
    puts(1, "\n  tag attribute must be in double(\") quotes \n")
    abort(1)
  end if

end procedure

global function SetUpFields(sequence tag, sequence screen)
  sequence field, endtag, temp, field_defaults
  integer fieldno, startrow, startcol, endcol, e1

  field_defaults = {0,0,0,1,0,WHITE,{}}

  --*----------------------------------------------------------------*
  --*  format of tag fields are:                                     *
  --*  { {"tag char", "attrib", color},                              *
  --*    {"tag char", "attrib", color},...                           *
  --*    "fieldendchar"   }                                          *
  --*  there may be many tag starting fields, but only one end char  *
  --*                                                                *
  --*  tag character can be any special character, as long as it     *
  --*    does not duplicate another special character on the screen  *
  --*  attrib is U - Unprotected or P - Protected                    *
  --*  color is the numeric value of the foreground color or the     *
  --*    defined color variables (i.e., BLUE, WHITE, etc.)           *
  --*----------------------------------------------------------------*

  fieldno = 0
  field   = {}
  e1      = length(tag)                                -- last element of tag
  endtag  = tag[e1]                                    -- is endtag char

  for i = 1 to length(screen) do
    temp = screen[i]                             -- save line for printing
    for j = 1 to length(tag) - 1 do              -- look for tag chars
      ValidateTag(tag[j][1])                     -- check tag character
      ValidateTag(tag[j][2])                     -- check protected attribute
      if match(tag[j][1], screen[i]) then        -- found tag char
        while match(tag[j][1], screen[i]) do     -- start of field
          fieldno += 1                           -- increment field number
          field = append(field, field_defaults)  -- set up field attributes
          startrow = i
          startcol = match(tag[j][1], screen[i]) + 1
          endcol   = match(endtag, screen[i])          -- end of field
          if endcol = 0 then                           -- missing end tag?
            text_color(BRIGHT_WHITE)
            puts(1, "\n  include screenio.e function SetUpFields: error\n")
            printf(1, "  end tag \'%s\' missing or mismatch in number of start and end tags on line %d\n",
                      {endtag, i})
            printf(1, "line contents:\n%s\n", {temp})
            text_color(WHITE)
            abort(1)
          elsif endcol < startcol then        -- end tag before start tag?
            text_color(BRIGHT_WHITE)
            puts(1, "\n  include screenio.e function SetUpFields: error\n")
            printf(1, "  end tag \'%s\' found before start tag on line %d\n",
                      {endtag, i})
            printf(1, "line contents:\n%s\n", {temp})
            abort(1)
            text_color(WHITE)
          end if
          field[fieldno][ROW]        = startrow          -- load field row
          field[fieldno][COL]        = startcol          -- load field col
          field[fieldno][FIELDLEN]   = endcol - startcol -- load field length
          field[fieldno][PROTECTED]  = match(tag[j][2], "UP") - 1
                                          -- Unprotected = 0, Protected = 1
          field[fieldno][FIELDCOLOR] = tag[j][3]       -- load field color
          endcol = endcol - 1
          field[fieldno][VALUE] = screen[i][startcol..endcol]  -- load field value
          screen[i][startcol - 1] = ' '                -- elim start tag
          screen[i][endcol + 1]   = ' '                -- elim end tag
        end while
      end if
    end for
  end for

  position(1, 1)                    -- start at top of screen
  for i = 1 to length(screen) do    -- print screen without tag chars
    puts(1, screen[i] & "\n")
  end for
  RefreshScreen(field)              -- print fields on screen

  return field

end function

global function SetUpFields2(sequence tag, sequence screen)
  sequence field, temp, field_defaults
  integer fieldno, startrow, startcol, endcol, s1

  field_defaults = {0,0,0,1,0,WHITE,{}}

  --*----------------------------------------------------------------*
  --*  This is different from SetUpFields in that there is no endtag *
  --*  character, and the field length is determined by the number   *
  --*  of consecutive special characters.  If there is a break in    *
  --*  special characters, that is the end of the field.             *
  --*                                                                *
  --*  Format of tag fields are:                                     *
  --*  { {"tag char", "attrib", color},                              *
  --*    {"tag char", "attrib", color},... }                         *
  --*  there may be many tag starting fields                         *
  --*  The length of the field is as long as there are consecutive   *
  --*  tag characters.                                               *
  --*  Take care that the tag character is not the same as another   *
  --*  special character on the screen.                              *
  --*  attrib is U - Unprotected or P - Protected                    *
  --*  color is the numeric value of the foreground color or the     *
  --*    defined color variables (i.e., BLUE, WHITE, etc.)           *
  --*----------------------------------------------------------------*

  fieldno = 0
  field   = {}

  for i = 1 to length(screen) do
    for j = 1 to length(tag) do                   -- look for tag chars
      ValidateTag(tag[j][1])                      -- tag character in "" ?
      ValidateTag(tag[j][2])                      -- protected attribute in "" ?
      if match(tag[j][1], screen[i]) then         -- found tag char
        while match(tag[j][1], screen[i]) do      -- start of field
          fieldno += 1                            -- increment field number
          field = append(field, field_defaults)   -- add field, attributes
          s1 = match(tag[j][1], screen[i])
          startrow = i
          startcol = s1
          field[fieldno][ROW]        = startrow    -- load field row
          field[fieldno][COL]        = startcol    -- load field col
          field[fieldno][FIELDLEN]   = 0
          field[fieldno][PROTECTED]  = match(tag[j][2], "UP") - 1
                                          -- Unprotected = 0, Protected = 1
          field[fieldno][FIELDCOLOR] = tag[j][3]       -- load field color
          while equal(screen[i][s1..s1], tag[j][1]) do
            field[fieldno][FIELDLEN] += 1
            screen[i][s1] = ' '                   -- blank out special char
            s1 += 1
            if s1 > length(screen[i]) then
              exit
            end if
          end while
          endcol = startcol + field[fieldno][FIELDLEN] - 1
          field[fieldno][VALUE] = screen[i][startcol..endcol]  -- load field value
        end while
      end if
    end for
  end for

  position(1, 1)                    -- start at top of screen
  for i = 1 to length(screen) do    -- print screen without tag chars
    puts(1, screen[i] & "\n")
  end for
  RefreshScreen(field)              -- print fields on screen

  return field

end function

global function ReadScreen(sequence fieldseq)
  sequence fields
  integer fieldno, anykey, fcol, s1, j, cursor_on_prot_field

  if atom(fieldseq[1][1]) then
    clear_screen()
    puts(1, "\n  include screenio.e function ReadScreen: error")
    puts(1, "\n  missing braces {} around fieldseq")
    puts(1, "\n  syntax is :")
    text_color(BRIGHT_GREEN)
    puts(1, "\n  sequence = ReadScreen({sequence, integer})")
    text_color(BRIGHT_WHITE)
    puts(1, "\n    OR")
    text_color(BRIGHT_GREEN)
    puts(1, "\n  sequence = ReadScreen({sequence})")
    text_color(BRIGHT_WHITE)
    puts(1, "\n  where integer is number of field to position cursor")
    abort(1)
  end if

  fields = fieldseq[1]

  if length(fieldseq) > 1 then      --sets cursor position
    fieldno = fieldseq[2]           --set cursor to field number specified
  else
    fieldno = 1                     --default cursor in field 1
  end if

  cursor_on_prot_field = 0
  if fields[fieldno][PROTECTED] then  --if starting field is protected
    cursor_on_prot_field = 1          -- move cursor to first unprotected
  end if                              -- field

  j = 0                             -- make sure there is at least
  while 1 do                        -- one unprotected field to read
    j = j + 1
    if j > length(fields) then
      puts(1, "\n  include screenio.e function ReadScreen: error\n")
      puts(1, "  Read error - No unprotected fields to read\n")
      abort(1)
    end if
    if not fields[j][PROTECTED] then
      if cursor_on_prot_field then   -- move cursor to unprotected
        fieldno = j                  -- field
      end if                         -- exit when an unprotected field
      exit                           -- is found
    end if
  end while

  for i = 1 to length(fields) do                            --make sure fields
    if length(fields[i][VALUE]) < fields[i][FIELDLEN] then  --are as long
      s1 = fields[i][FIELDLEN] - length(fields[i][VALUE])   --as they should be
      fields[i][VALUE] = fields[i][VALUE] & repeat(' ', s1) --pad with blanks
    end if                                                  --if they are not
  end for

  SetCursor(fields[fieldno][ROW], fields[fieldno][COL])

  while 1 do
    anykey = wait_key()

    if anykey = ESC then
      return {}
    elsif anykey = EnterKey then
      ResetCursor(fields[fieldno][ROW],
                  fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      for i = 1 to length(fields) do    --reset all current cols to pos 1
        fields[i][CURRCOL] = 1
      end for

      return fields
    elsif anykey = TabKey then
      ResetCursor(fields[fieldno][ROW],
                  fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      while 1 do                                 -- go to next
        fieldno = fieldno + 1                    -- unprotected field
        if fieldno > length(fields) then
          fieldno = 1
        end if
        if not fields[fieldno][PROTECTED] then   -- if unprotected field,exit
          exit
        end if
      end while
      text_color(fields[fieldno][FIELDCOLOR])
      SetCursor(fields[fieldno][ROW],
                fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
    elsif anykey = BackTab then
      ResetCursor(fields[fieldno][ROW],
                  fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      while 1 do                                 -- go to prev
        fieldno = fieldno - 1                    -- unprotected field
        if fieldno < 1 then
          fieldno = length(fields)
        end if
        if not fields[fieldno][PROTECTED] then   -- if unprotected field,exit
          exit
        end if
      end while
      text_color(fields[fieldno][FIELDCOLOR])
      SetCursor(fields[fieldno][ROW],
                fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
    elsif anykey = Backspace or anykey = LeftArrow then
      ResetCursor(fields[fieldno][ROW],
                  fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      if fields[fieldno][CURRCOL] > 1 then
        fields[fieldno][CURRCOL] -= 1
      end if
      SetCursor(fields[fieldno][ROW],
                fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
    elsif anykey = RightArrow then
      ResetCursor(fields[fieldno][ROW],
                  fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      if fields[fieldno][CURRCOL] < fields[fieldno][FIELDLEN] then
        fields[fieldno][CURRCOL] += 1
      end if
      SetCursor(fields[fieldno][ROW],
                  fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
    elsif anykey > 31 and anykey < 128 then
      ResetCursor(fields[fieldno][ROW],
                fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      text_color(fields[fieldno][FIELDCOLOR])
      position(fields[fieldno][ROW],
               fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
      puts(1, anykey)
      fcol = fields[fieldno][CURRCOL]
      fields[fieldno][VALUE][fcol] = anykey
      if fields[fieldno][CURRCOL] < fields[fieldno][FIELDLEN] then
        fields[fieldno][CURRCOL] += 1
      end if
      SetCursor(fields[fieldno][ROW],
                fields[fieldno][COL] + fields[fieldno][CURRCOL] - 1)
    end if

  end while

end function

