  include get.e
  include graphics.e
  include keys.e
  include fileio.e
  include screenio.e

  --*----------------------------------------------------------------*
  --*  Demo payment calculator program                               *
  --*  demonstrates how to use fields                                *
  --*----------------------------------------------------------------*

  constant PRIN = 1, INTRATE = 2, MONTHS = 3, MTHLY_PMT = 4, ERRMSG = 5

  sequence screen, tag, fields,
           principal_x, int_rate_x, months_x
  atom     principal, int_rate, months, monthly_pmt
  integer errors, fieldno

  tag = {
         {"@", "U", BRIGHT_WHITE},
         {"%", "P", RED}
        }

  screen =
  {
   "            Calculate Monthly Payments                       ",
   "                                                             ",
   "     Principal              @@@@@@@@@@ (enter 0 to exit)     ",
   "     Interest Rate               @@@@@                       ",
   "     Months                       @@@@                       ",
   "     Monthly Payment          @@@@@@@@                       ",
   " %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  "
  }

function Calculate_interest(atom principal, atom int_rate, integer months)
  atom monthly_pmt, temp_int, temp_int2

  temp_int    = int_rate / 1200
  temp_int2   = (1 - power((1 + temp_int), -months)) / temp_int
  monthly_pmt = principal / temp_int2

  return monthly_pmt

end function

  text_color(GREEN) bk_color(BLUE)
  clear_screen()

  fields  = SetUpFields2(tag, screen)
  fieldno = PRIN

  while 1 do                         -- loop until principal is zero
    while 1 do                       -- if there are no errors, this will exit
      errors        = 0              -- set errors to 0
      fields        = ReadScreen({fields, fieldno})
      principal_x   = RTrim(fields[PRIN]     [VALUE])
      int_rate_x    = RTrim(fields[INTRATE]  [VALUE])
      months_x      = RTrim(fields[MONTHS]   [VALUE])
      principal     = VAL(principal_x)
      int_rate      = VAL(int_rate_x)
      months        = VAL(months_x)

      if principal = 0 then          -- end program if principal is zero
        exit
      end if

      if principal < 0 or principal > 999999 then  -- validate principal
        errors                     = 1
        fieldno                    = PRIN
        fields[ERRMSG][VALUE]      = "Principal must be 0 - 999999"
        fields[PRIN]  [FIELDCOLOR] = RED
        RefreshScreen(fields)
      end if

      if int_rate <= 0 or int_rate > 100 then      -- validate interest rate
        errors                      = 1
        fieldno                     = INTRATE
        fields[ERRMSG] [VALUE]      = "Interest rate must be 1 - 100"
        fields[INTRATE][FIELDCOLOR] = RED
        RefreshScreen(fields)
      end if

      if months <= 0 or months > 300 then          -- validate months
        errors                     = 1
        fieldno                    = MONTHS
        fields[ERRMSG][VALUE]      = "Months must be 1 - 300"
        fields[PRIN]  [FIELDCOLOR] = RED
        RefreshScreen(fields)
      end if

      if errors = 0 then
        exit
      end if

    end while

    if principal = 0 then            -- end program if principal is zero
      exit
    end if

    monthly_pmt              = Calculate_interest(principal, int_rate, months)
    fields[MTHLY_PMT][VALUE] = sprintf("%7.2f", monthly_pmt)
    RefreshScreen(fields)

  end while

  position(21,1)
  puts(1, "Program ended.\n")

