--*------------------------------------------------------------------*
--*  DISPRECS.E                                                      *
--*  Display records with scrolling, ESC to return                   *
--*------------------------------------------------------------------*

  without warning

  include get.e
  include image.e

  constant DFLT_FGCOLOR = WHITE, DFLT_BGCOLOR = BLUE,
           Default_Screenprint_File = "c:\\mydocu~1\\temptxt"

  sequence toplit, bottomlit, ScaleLine, ScaleLine2, saved_cursor
  integer EBCDIC_mode
  toplit    = "......  " & repeat('=', 32) & " T O P " & repeat('=', 33)
  bottomlit = "......  " & repeat('=', 29) & " B O T T O M " & repeat('=', 30)
  ScaleLine = "......  <...+....1....+....2....+....3....+....4....+....5....+....6....+....7.>"
  ScaleLine2 = "....+....1....+....2....+....3....+....4....+....5....+....6" &
               "....+....7....+....8....+....9....+....0....+....1....+....2" &
               "....+....3....+....4....+....5....+....6....+....7.."

procedure Initialize()
  sequence StatusLine
  integer HiLite1, HiLite2, HiLite3, HiLite4, HiLite5, HiLite6, HiLite7

  StatusLine = " F3 Hex On/Off  F4 Top  F5 Bottom  F6 EBCDIC F10 Left  F11 Right  ESC Exit "
  HiLite1 = match("F3",   StatusLine)
  HiLite2 = match("F4",   StatusLine)
  HiLite3 = match("F5",   StatusLine)
  HiLite4 = match("F6",   StatusLine)
  HiLite5 = match("F10",  StatusLine)
  HiLite6 = match("F11",  StatusLine)
  HiLite7 = match("ESC",  StatusLine)

  text_color(WHITE)  bk_color(BLUE)
  clear_screen()

  text_color(BLACK)   bk_color(CYAN)
  position(24,1)  puts(1, repeat(' ', 80))
  position(24,1)  puts(1, StatusLine)
  text_color(BRIGHT_WHITE)
  position(24, HiLite1) puts(1, "F3")
  position(24, HiLite2) puts(1, "F4")
  position(24, HiLite3) puts(1, "F5")
  position(24, HiLite4) puts(1, "F6")
  position(24, HiLite5) puts(1, "F10")
  position(24, HiLite6) puts(1, "F11")
  position(24, HiLite7) puts(1, "ESC")

  scroll(-1, 24, 25)
  text_color(WHITE)  bk_color(BLUE)
  position(24,1)  puts(1, repeat(' ', 80))

end procedure

procedure ResetCursor(integer row, integer col)
  integer cchar

  display_text_image({row, col}, saved_cursor)
  cchar = saved_cursor[1][1]

  if cchar = '' then
    position(row, col) puts(1, ' ')
  end if

end procedure

procedure SetCursor(integer row, integer col)
  integer cchar, cfore, cback

  position(row, col)
  saved_cursor = save_text_image({row, col}, {row, col})
  cchar = saved_cursor[1][1]
  cfore = remainder(saved_cursor[1][2], 16)
  cback = floor(saved_cursor[1][2] / 16)

  if cchar = ' ' or cchar = '' then
    text_color(cfore) bk_color(cback)
    position(row, col) puts(1, '')
  else
    text_color(cback) bk_color(cfore)
    position(row, col) puts(1, cchar)
  end if

  position(row, col)
  text_color(DFLT_FGCOLOR) bk_color(DFLT_BGCOLOR)

end procedure

procedure Screenprint()
  sequence temprow, screen, save_screen1

  save_screen1 = save_text_image({1, 1}, {24, 80})
  screen = {}

  for irow = 1 to 24 do
    temprow = {}
    for j = 1 to 159 by 2 do
      temprow = temprow & save_screen1[irow][j]
    end for
    screen = append(screen, temprow)
  end for

  WriteTextFile({Default_Screenprint_File", screen})
  MsgBox("file " & Default_Screenprint_File & " written")
end procedure

procedure DisplayRecs(sequence line_array, integer topline, integer LeftMargin, integer hexmode)
  sequence temp1, temp2, temp2a, temp3, temp3a, temp3b
  integer  startmargin, firstline, lineno, irow

  if hexmode = 3 then             -- hexmode 3 = EBCDIC character mode
    EBCDIC_mode = 1
    hexmode = 0
  elsif hexmode = 4 then          -- hexmode 4 = ECBDIC hex mode
    EBCDIC_mode = 1
  end if

  --*----------------------------------------------------------------*
  --*  Display left margin and scale line                            *
  --*----------------------------------------------------------------*

  if LeftMargin > 1 then
    temp1 = sprintf("%05d", LeftMargin)
    temp2 = value(temp1[4..5])             -- get last 2 digits for offset
    startmargin = temp2[2]                 -- from ScaleLine2
    if startmargin = 0 then
      startmargin = 100
    end if
    position (6, 1) puts(1, "......  " & ScaleLine2[startmargin..startmargin+71])
  else
    position (6, 1) puts(1, ScaleLine)
  end if

  text_color(BRIGHT_RED)
  position(5, 32) printf(1, "COLS  %05d %05d",{LeftMargin, LeftMargin + 71})
  text_color(WHITE)

  if topline = 1 then
    position(7, 1)  puts(1, toplit)
    firstline = 8
  else
    firstline = 7
  end if

  --*----------------------------------------------------------------*
  --*  Display lines in character and/or hex                         *
  --*----------------------------------------------------------------*
  lineno = topline
  irow = firstline

  while irow < 25 do
    if lineno > length(line_array) or (hexmode and irow + 3 > 24) then
      if lineno > length(line_array) then
        position(irow, 1) puts(1, bottomlit)       -- no more lines
      else
        position(irow, 1) puts(1, repeat(' ', 80))
      end if
      for j = irow + 1 to 24 do                  -- blank out rest of screen
        position(j, 1) puts(1, repeat(' ', 80))
      end for
      exit
    end if

    temp1 = sprintf("%06d", lineno)
    temp2 = line_array [lineno]
    if LeftMargin > length(temp2) then
      temp2 = repeat(' ', 72)
    else
      temp2 = temp2 [LeftMargin..length(temp2)]
    end if

    if length(temp2) > 72 then         -- if line longer than 72
      temp2 = temp2[1..72]             -- truncate to 72 chars for print
    end if

    if EBCDIC_mode then
      temp2a = EBCDIC_to_ASCII(temp2)
    end if

    temp3 = {}
    temp3a = {}
    temp3b = {}
    position(irow, 1) puts(1, repeat(' ', 80))  -- clear out line

    for j = 1 to length(temp2) do
      if hexmode then
        temp3 = sprintf("%02x", temp2[j])
        temp3a &= temp3[1]
        temp3b &= temp3[2]
      end if
      if temp2[j] < 32 or temp2[j] > 127 then
        temp2[j] = '.'
      end if
    end for

    if EBCDIC_mode then
      position(irow, 1) puts(1, temp1 & "  " & temp2a)
    else
      position(irow, 1) puts(1, temp1 & "  " & temp2)
    end if

    if hexmode then
      position(irow + 1, 1) puts(1, repeat(' ', 80))
      position(irow + 2, 1) puts(1, repeat(' ', 80))
      position(irow + 1, 9) puts(1, temp3a)
      position(irow + 2, 9) puts(1, temp3b)
      irow   = irow + 2
    end if

    lineno = lineno + 1
    irow   = irow + 1
  end while

end procedure

global procedure Display(sequence line_array)
  integer anykey, currline, LeftMargin, hexmode, row, col
  sequence numlines
  object line

  line = line_array
  if atom(line[1]) then        -- if line_array is a string
    line_array = {line_array}  -- convert to a sequence
  end if

  row = 1 col = 1
  numlines    = {16, 3}        -- numlines if hex off/hex on
  currline    = 1
  LeftMargin  = 1
  hexmode     = 0
  EBCDIC_mode = 0
  Initialize()
  DisplayRecs(line_array, currline, LeftMargin, hexmode)
  SetCursor(row, col)

  while 1 do
    anykey = wait_key()
    if anykey = ESC then
      exit
    elsif find(anykey, {LeftArrow, RightArrow, UpArrow, DownArrow}) then
      ResetCursor(row, col)
      if anykey = LeftArrow then
        col = col - 1
      elsif anykey = RightArrow then
        col = col + 1
      elsif anykey = UpArrow then
        row = row - 1
      elsif anykey = DownArrow then
        row = row + 1
      end if
      if col < 1 then
        col = 80
      elsif col > 80 then
        col = 1
      end if
      if row < 1 then
        row = 24
      elsif row > 24 then
        row = 1
      end if
      SetCursor(row, col)
    elsif anykey = PgDn then
      if row > 7 and not hexmode then
        currline = currline + (row - 8) + (currline > 1)
      else
        currline = currline + numlines[hexmode + 1] + (currline > 1)
      end if
      if currline > length(line_array) then
        currline = length(line_array) - numlines[hexmode + 1]
        if currline < 1 then
          currline = 1
        end if
      end if
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = PgUp then
      currline = currline - numlines[hexmode + 1] - 1
      if currline < 1 then
        currline = 1
      end if
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = HomeKey then
      row = 1  col = 1
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = F3 then
      hexmode = 1 - hexmode                  -- toggle hex on/off
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = F4 then
      currline = 1
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = F5 then
      currline = length(line_array) - numlines[hexmode + 1]
      if currline < 1 then
        currline = 1
      end if
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = F6 then
      hexmode = 1 - hexmode                  -- toggle EBCDIC hex on/off
      if hexmode then
        DisplayRecs(line_array, currline, LeftMargin, 4)
      else
        DisplayRecs(line_array, currline, LeftMargin, 3)
      end if
      SetCursor(row, col)
    elsif anykey = F10 then
      LeftMargin = LeftMargin - 72
      if LeftMargin < 1 then
        LeftMargin = 1
      end if
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = F11 then
      if col > 9 then
        LeftMargin = LeftMargin + col - 9
      else
        LeftMargin = LeftMargin + 72
      end if
      DisplayRecs(line_array, currline, LeftMargin, hexmode)
      SetCursor(row, col)
    elsif anykey = F12 then            -- screen print
      Screenprint()
    end if
  end while

  return

end procedure

global procedure DisplayText(sequence search_text, sequence line_array)
  sequence new_array

  if length(line_array) = 0 then
    return
  end if

  new_array = {}
  for i = 1 to length(line_array) do
    if match(search_text, line_array[i]) then
      new_array = append(new_array, line_array[i])
    end if
  end for

  Display(new_array)

end procedure

global function ExtractText(sequence search_text, sequence line_array)
  sequence new_array

  if length(line_array) = 0 then
    return {}
  end if

  new_array = {}
  for i = 1 to length(line_array) do
    if match(search_text, line_array[i]) then
      new_array = append(new_array, line_array[i])
    end if
  end for

  return new_array

end function

global function DropDuplicates(sequence line_array)
  sequence new_array, prev_rec


  if length(line_array) = 0 then
    return {}
  end if

  new_array = {}
  prev_rec = line_array[1]

  for i = 2 to length(line_array) do
    if not equal(line_array[i], prev_rec) then
      new_array = append(new_array, line_array[i])
      prev_rec = line_array[i]
    end if
  end for

  return new_array

end function

