
--*------------------------------------------------------------------*
--*  fileio.e                                                        *
--*  routines for opening, reading and writing text or binary files  *
--*------------------------------------------------------------------*

  without warning

  include get.e
  include file.e
  include image.e
  include wildcard.e
  include graphics.e
  include drawbox.e
  constant TRUE = 1, FALSE = 0

global function OpenFile(sequence filename, sequence filetype)
  sequence filedesc
  integer fn

  filetype = lower(filetype)
  if equal(filetype, "r") then
    filedesc = "input"
  elsif equal(filetype, "rb") then
    filedesc = "binary input"
  elsif equal(filetype, "w") then
    filedesc = "output"
  elsif equal(filetype, "wb") then
    filedesc = "binary output"
  elsif equal(filetype, "a") then
    filedesc = "append"
  elsif equal(filetype, "ab") then
    filedesc = "append binary"
  elsif equal(filetype, "u") then
    filedesc = "update"
  elsif equal(filetype, "ub") then
    filedesc = "binary update"
  end if

  fn = open(filename, filetype)
  if fn = -1 then
    printf(1, "cannot open %s for %s\n", {filename, filedesc})
    puts(1, " Press any key\n")
    fn = wait_key()
    abort(1)
  end if

  return fn

end function

global function ReadTextFile(integer fn)
  object line
  integer EOF, lastchar

  EOF = FALSE

  line = gets(fn)
  if sequence(line) then                  -- not EOF yet
    lastchar = length(line)
    if line[lastchar] = '\n' then         -- strip off newline char
      line = line[1..lastchar-1]
    end if
  else                                    -- EOF is an atom (-1)
    EOF = TRUE
    line = {}
  end if

  return {EOF, line}

end function

global function LoadTextFile(sequence filename)
  object line
  sequence line_array
  integer fn, lastchar

  fn = open(filename, "r")
  if fn = -1 then
    return {}
  end if

  line_array = {}

  line = gets(fn)
  while sequence(line) do                 -- EOF is an atom (-1)
    lastchar = length(line)
    if line[lastchar] = '\n' then         -- strip off newline char
      line = line[1..lastchar-1]
    end if
    line_array = append(line_array, line)
    line = gets(fn)
  end while

  close(fn)

  return line_array

end function

global function LoadPartialTextFile(sequence parm)
  object line
  sequence filename, line_array
  integer fn, recs_read, startrec, endrec, lastchar

  filename = parm[1]
  if length(parm) = 2 then
    startrec = 1
    endrec   = parm[2]
  elsif length(parm) = 3 then
    startrec = parm[2]
    endrec   = parm[3]
  else
    puts(1, "Error in fileio.e: function LoadPartialTextFile - Arguments must be\n")
    puts(1, "FILENAME <nrecs> or FILENAME <startrec> <endrec>\n")
    abort(1)
  end if

  recs_read = 0
  fn = open(filename, "r")
  if fn = -1 then
    return {}
  end if

  line_array = {}
  line = gets(fn)
  while sequence(line) do                 -- EOF is an atom (-1)
    recs_read += 1

    if recs_read > endrec then
      exit
    elsif recs_read >= startrec then
      lastchar = length(line)
      if line[lastchar] = '\n' then       -- strip off newline char
        line = line [1..lastchar-1]
      end if
      line_array = append(line_array, line)
    end if
    line = gets(fn)

  end while

  close(fn)
  return line_array

end function

global function ReadBinaryFile_Seq(integer fn, integer LRECL)
  sequence line
  integer EOF

  EOF = FALSE

  line = get_bytes(fn, LRECL)
  if length(line) < LRECL then
    EOF = TRUE
  end if

  return {EOF, line}

end function

global function LoadBinaryFile(sequence filename, integer LRECL)
  sequence line, line_array
  integer fn

  fn = open(filename, "rb")
  if fn = -1 then
    return {}
  end if

  line_array = {}
  line = get_bytes(fn, LRECL)

  while length(line) = LRECL do
    line_array = append(line_array, line)
    line       = get_bytes(fn, LRECL)
  end while

  if length(line) > 0 then                -- EOF, any data left over?
    line_array = append(line_array, line) -- add to file
  end if

  close(fn)

  return line_array

end function

function FileExist(sequence out_Filename)
  object out_Filename_dir
  sequence saved_screen_area, OK_To_Save, prompt_msg

  out_Filename_dir = dir(out_Filename) -- look for directory entry
  if sequence(out_Filename_dir) then   -- found it
    saved_screen_area = save_text_image({9, 2}, {12, 75})  -- save area on screen
    DrawBox({9, 3, 11, 74, CYAN, BLUE, 1})                 -- draw box
    text_color(BRIGHT_WHITE)
    position(10, 4)
    prompt_msg = " File " & out_Filename &
                 " already exists. Overwrite(y/n/a) "
    OK_To_Save = prompt_string(prompt_msg)
    display_text_image({9, 2}, saved_screen_area)  -- restore screen
    text_color(WHITE)  bk_color(BLUE)              -- behind box
    if match(upper(OK_To_Save), "A") then          -- abort
      abort(1)
    end if
    if match(upper(OK_To_Save), "Y") then          -- overwrite
      return FALSE
    else
      return TRUE                                  -- do not overwrite
    end if
  else                                             -- file does not exist
    return FALSE
  end if

end function

global function Resize(sequence line_array, integer LRECL)
  integer String_passed
  String_passed = FALSE

  if length(line_array) = 0 then              --no records
    return {}
  end if

  if atom(line_array[1]) then      -- convert string to 1 element sequence
    line_array = {line_array}
    String_passed = TRUE
  end if

  for i = 1 to length(line_array) do
    if length(line_array[i]) < LRECL then     --pad w/blanks if shorter
      line_array[i] &= repeat(' ', LRECL - length(line_array[i]))
    end if
    if length(line_array[i]) > LRECL then     --trim if longer
      line_array[i] = line_array[i][1..LRECL]
    end if
  end for

  if String_passed then
    return line_array[1]                  -- convert sequence back to string
  else
    return line_array
  end if

end function

global procedure WriteBinaryFile(sequence parm)
  sequence filename, line_array, filemode
  integer fn

  if length(parm) = 0 then
    puts(1, "<fileio.e : procedure WriteBinaryFile> No file name specified\n")
    abort(1)
  end if

  filename      = parm[1]
  line_array    = parm[2]
  filemode      = "wb"

  if length(parm) > 2 then
    filemode = parm[3]
  end if

  if not FileExist(filename) then
    fn = OpenFile(filename, filemode)

    for i = 1 to length(line_array) do
      for j = 1 to length(line_array[i]) do
        puts(fn, line_array[i][j])
      end for
    end for

    close(fn)

  end if

end procedure

global procedure WriteTextFile(sequence parm)
  sequence filename, line_array, filemode
  integer fn

  if length(parm) = 0 then
    puts(1, "No file name specified\n")
    abort(1)
  end if

  filename      = parm[1]
  line_array    = parm[2]
  filemode      = "w"

  if atom(line_array[1]) then
    line_array = {line_array}         -- convert string to 1-element sequence
  end if

  if length(parm) > 2 then
    filemode = parm[3]
  end if

  if not FileExist(filename) then
    fn = OpenFile(filename, filemode)

    for i = 1 to length(line_array) do
      puts(fn, line_array[i] & "\n")
    end for

    close(fn)
  end if

end procedure

global function Unpack(sequence packed_field, integer numdec)
  sequence unpacked_field
  integer  lastchar

  unpacked_field = {}

  for i = 1 to length(packed_field) do
    unpacked_field = unpacked_field & sprintf("%02x", packed_field[i])
  end for

  lastchar = length(unpacked_field)
  if unpacked_field[lastchar] = 'C' then
    unpacked_field = '+' & unpacked_field[1..lastchar-1]
  else
    unpacked_field = '-' & unpacked_field[1..lastchar-1]
  end if

  if numdec > 0 then
    unpacked_field = unpacked_field[1..lastchar-numdec] & '.' &
                     unpacked_field[lastchar-numdec+1..lastchar]
  end if

  return unpacked_field

end function

function Init_ascii_table()
  sequence ascii_table

  -- A0 = 160
  -- B0 = 176
  -- C0 = 192
  -- D0 = 208
  -- E0 = 224
  -- F0 = 240

  ascii_table = repeat('.', 255)
  ascii_table [64] = ' '
  ascii_table [75..78]   = ".<(+"
  ascii_table [80]       = '&'
  ascii_table [90..97]   = "!$*);^-/"
  ascii_table [106..111] = "|,%_>?"
  ascii_table [121..127] = "`:#@\'=\""
  ascii_table [129..137] = "abcdefghi"
  ascii_table [145..153] = "jklmnopqr"
  ascii_table [161..169] = "~stuvwxyz"
  ascii_table [173]      = '['
  ascii_table [189]      = ']'
  ascii_table [192..201] = "{ABCDEFGHI"
  ascii_table [208..217] = "}JKLMNOPQR"
  ascii_table [224..224] = "\\"
  ascii_table [226..233] = "STUVWXYZ"
  ascii_table [240..249] = "0123456789"

  return ascii_table

end function

function Init_EBCDIC_table()
  sequence EBCDIC_table

   EBCDIC_table = {
   001, 002, 003, 055, 045, 046, 047, 022, 005, 037,        --  01 -  10
   011, 012, 013, 014, 015, 016, 017, 018, 000, 060,        --  11 -  20
   061, 050, 038, 024, 025, 063, 039, 034, 029, 030,        --  21 -  30
   031, 064, 090, 127, 123, 091, 108, 080, 125, 077,        --  31 -  40
   093, 092, 078, 107, 096, 075, 097, 240, 241, 242,        --  41 -  50
   243, 244, 245, 246, 247, 248, 249, 122, 094, 076,        --  51 -  60
   126, 110, 111, 124, 193, 194, 195, 196, 197, 198,        --  61 -  70
   199, 200, 201, 209, 210, 211, 212, 213, 214, 215,        --  71 -  80
   216, 217, 226, 227, 228, 229, 230, 231, 232, 233,        --  81 -  90
   173, 224, 189, 095, 109, 121, 129, 130, 131, 132,        --  91 - 100
   133, 134, 135, 136, 137, 145, 146, 147, 148, 149,        -- 111 -  10
   150, 151, 152, 153, 162, 163, 164, 165, 166, 167,        --  11 -  20
   168, 169, 192, 106, 208, 161, 007, 000, 000, 000,        --  21 -  30
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  31 -  40
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  41 -  50
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  51 -  60
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  61 -  70
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  71 -  80
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  81 -  90
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  91 - 200
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        -- 201 -  10
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  11 -  20
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  21 -  30
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  31 -  40
   000, 000, 000, 000, 000, 000, 000, 000, 000, 000,        --  41 -  50
   000, 000, 000, 000, 000}                                 --  51 -  55

  return EBCDIC_table

end function

global function EBCDIC_to_ASCII(sequence line_array)
  sequence ascii_table, temp
  integer sub, space, File_converted

  File_converted = FALSE

  if length(line_array) = 0 then
    return {}
  end if

  if atom(line_array[1]) then   --this code is here
    temp = line_array           --so that one record can be
  else                          --passed or a
    temp = line_array[1]        --series of records
  end if

  space = 0
  for i = 1 to length(temp) do
    if temp[i] = ' ' then
      space = space + 1
    end if
  end for

  if space < 10 then         --if there are at least 10 ASCII spaces
    File_converted = FALSE   --file is assumed to be converted,
  else                       --so we don't try to convert file again
    File_converted = TRUE
  end if

  if File_converted then
    return line_array
  end if

  ascii_table = Init_ascii_table()

  for i = 1 to length(line_array) do
    if atom(line_array[i]) then    --if this is a single line
      sub = line_array[i]          --do this
      if sub = 0 then
        line_array[i] = 32         -- convert low value to space
      else
        line_array[i] = ascii_table[sub]
      end if
    else                           --else this is part of an array
      for j = 1 to length(line_array [i]) do
        sub = line_array [i][j]
        if sub = 0 then
          line_array [i][j] = 32     --convert low value to space
        else
          line_array [i][j] = ascii_table[sub]
        end if
      end for
    end if
  end for

  return line_array

end function

global function ASCII_to_EBCDIC(sequence line_array)
  sequence EBCDIC_table, temp
  integer sub

  if length(line_array) = 0 then
    return {}
  end if

  if atom(line_array[1]) then   --this code is here
    temp = line_array           --so that one record can be
  else                          --passed or a
    temp = line_array[1]        --series of records
  end if

  EBCDIC_table = Init_EBCDIC_table()

  for i = 1 to length(line_array) do
    if atom(line_array[i]) then    --if this is a single line
      sub = line_array[i]          --do this
      if sub = 0 then
        line_array[i] = 0
      else
        line_array[i] = EBCDIC_table[sub]
      end if
    else                           --else this is part of an array
      for j = 1 to length(line_array [i]) do
        sub = line_array [i][j]
        if sub = 0 then
          line_array [i][j] = 0
        else
          line_array [i][j] = EBCDIC_table[sub]
        end if
      end for
    end if
  end for

  return line_array

end function

global function ConvertAmount(sequence amount, integer numdec)
  sequence number, converted_number
  integer lastchar, sign

  number   = "0123456789"
  lastchar = length(amount)

  sign = match(amount[lastchar..lastchar], "{ABCDEFGHI")
  if sign > 0 then
    amount = " " & amount
    lastchar += 1                               -- adjust length for sign
    amount[lastchar] = number[sign]
  else                                          -- not found, neg number?
    sign = match(amount[lastchar..lastchar], "}JKLMNOPQR")  -- match one of these?
    if sign > 0 then                            -- yes, it is a neg number
      amount = "-" & amount
      lastchar += 1                             -- adjust length for sign
      amount[lastchar] = number[sign]
    else
      amount = " " & amount
      lastchar += 1                             -- adjust length for sign
      amount[lastchar] = '0'                    -- not found, default to 0
    end if
  end if

  if numdec > 0 then
    converted_number = amount[1..lastchar-numdec] & "." &
                       amount[lastchar-numdec+1..lastchar]
  else
    converted_number = amount
  end if

  return converted_number

end function

global function CommaFormat(sequence rawnum)
  sequence fmtnum
  integer s1, sign

  if rawnum[1] = '+' or rawnum[1] = '-' then
    sign = 1
  else
    sign = 0
  end if

  s1 = match(".", rawnum)
  if s1 = 0 then                --if number has no decimal point,
    rawnum = rawnum & "."       --add one to the end of the number
  end if
  s1 = match(".", rawnum)
  if s1 < 5 + sign then
    if rawnum[length(rawnum)] = '.' then        --get rid of dec pt at end
      rawnum = rawnum[1..length(rawnum) - 1]    --of string
    end if
    return rawnum
  end if

  if s1 > 4 then
    s1 = s1 - 3
    while s1 > 1 + sign do
      fmtnum = rawnum[1..s1 - 1] & "," & rawnum[s1..length(rawnum)]
      rawnum = fmtnum
      s1 = s1 - 3
    end while
  end if

  if fmtnum[length(fmtnum)] = '.' then        --get rid of dec pt at end
    fmtnum = fmtnum[1..length(fmtnum) - 1]    --of string
  end if

  return fmtnum

end function

global function VAL(sequence string)
  sequence temp
  temp = value(string)
  return temp[2]

end function

global function RTrim(sequence string)
  integer lastchar
  lastchar = length(string)
  if lastchar > 0 then
    while lastchar > 0 do
      if string[lastchar] != ' ' then
        exit
      end if
      lastchar -= 1
    end while
    string = string[1..lastchar]
  end if

  return string

end function

global function LTrim(sequence string)
  integer currchar, orig_len

  if length(string) = 0 or string[1] != ' ' then
    return string
  end if

  currchar = 1
  while string[currchar] != ' ' and currchar < length(string) do
    currchar += 1
  end while

  if currchar = length(string) and string[currchar] = ' ' then  -- last char in string is space
    return {}
  end if

  string = string[currchar..length(string)]

  return string

end function

global function Trim(sequence string)
  integer currchar

  -- first trim leading spaces
  currchar = 1
  while string[currchar] != ' ' and currchar < length(string) do
    currchar += 1
  end while

  if currchar = length(string) and string[currchar] = ' ' then
    return {}
  end if

  string = string[currchar..length(string)]

  -- trim trailing spaces
  currchar = length(string)
  if currchar > 0 then
    while currchar > 0 do
      if string[currchar] != ' ' then
        exit
      end if
      currchar -= 1
    end while
  end if

  if length(string) = 0 then
    return {}
  end if

  string = string[1..currchar]

  return string

end function

global function prompt_password(sequence prompt)
  sequence s, string
  integer row, col, anykey

  puts(1, prompt)
  s = get_position()                      -- get cursor position
  row = s[1]                              -- get row
  col = s[2]                              -- get col
  position(row, col)
  string = {}

  while 1 do
    anykey = wait_key()
    if anykey = 27 then                   -- ESC, abort program
      abort(1)
    elsif anykey = 13 then                -- Enter key, return string
      return string
    elsif anykey = 8 then                 -- Backspace
      if length(string) > 0 then
        string = string[1..length(string) - 1]
        col -= 1
        position(row, col) puts(1, " ")
        position(row, col)
      end if
    elsif anykey > 30 and anykey < 128 then
      string &= anykey
      position(row, col)   puts(1, "*")
      col += 1
    end if
  end while

end function

global function parse(sequence parms)
  sequence string, result, part
  integer delimiter

  delimiter = ' '
  string = parms[1]
  if length(parms) > 1 then
    delimiter = parms[2]
  end if

  result = ""
  part = ""

  for i = 1 to length(string) do
    if string[i] = delimiter then
      result = append(result, part)
      part = ""
    else
      part &= string[i]
    end if
  end for

  if length(part) > 0 then           -- any string left over?
    result = append(result, part)    -- add to result
  end if

return result

end function

global function Numeric(sequence text)
  integer IsNumeric, NotNumeric, no_of_periods

  IsNumeric = 1  NotNumeric = 0
  no_of_periods = 0

  if length(text) = 0 then
    return NotNumeric
  end if

  for i = 1 to length(text) do
    if not find(text[i], "+-0123456789.") then
      return NotNumeric
    end if
    if text[i] = '.' then
      no_of_periods += 1
    end if
  end for

  if no_of_periods > 1 then
    return NotNumeric
  end if

  return IsNumeric

end function

